#include <windows.h>
#include <math.h>
#include <string.h>

/*
Written by Kenneth E. Moore
June 2, 2000
Extended to Z^4 and some bugs fixed June 4, 2001
*/

int __declspec(dllexport) APIENTRY UserGrinDefinition(double *data);
int __declspec(dllexport) APIENTRY UserParamNames(char *data);

BOOL WINAPI DllMain (HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
	{
   return TRUE;
   }

/* the data is stored as follows:
ndata = the number of elements in the array data
data[0]  = the total number of values in the passed data array
data[1]  = x
data[2]  = y
data[3]  = z
data[4]  = wavelength in microns
data[5]  = millimeters per unit length (1.0 for mm, 25.4 for inches, 10.0 for cm and 1000.0 for meters)
data[6]  = n (to be computed by the dll and returned)
data[7]  = n * dn/dx (to be computed by the dll and returned)
data[8]  = n * dn/dy (to be computed by the dll and returned)
data[9]  = n * dn/dz (to be computed by the dll and returned)
data[10] = parameter 1 from user input
data[11] = parameter 2 from user input
etc... up to data[maxdata] where maxdata = int(data[0])

The DLL must compute data 6 through 9 given the other data,
and return 0 if it works; else return -1
*/

/* this DLL models a GRIN media similar to the GRIN 5 surface type. */

int __declspec(dllexport) APIENTRY UserGrinDefinition(double *data)
	{
   double x, y, z, r2, N1, N2, index, temp;

   x = data[1];
   y = data[2];
   z = data[3];
   r2 = x*x + y*y;

   N1 = data[13];
	N1 = data[12] + r2*N1;
	N1 = data[11] + r2*N1;
	N1 = data[10] + r2*N1;
   
	N2 = data[17];
	N2 = data[16] + z*N2;
	N2 = data[15] + z*N2;
	N2 = data[14] + z*N2;
	N2 *= z;

	index = N1+N2;
	data[6] = index;

   if (index < 1.0)
		{
		return -1;
		}
	
	/* now the derivatives */
	temp = (2.0*data[11] + 4.0*data[12]*r2 + 6.0*data[13]*r2*r2);
   data[7] = index*temp*x;
   data[8] = index*temp*y;
   data[9] = index*(data[14] + 2.0*data[15]*z + 3.0*data[16]*z*z + 4.0*data[17]*z*z*z);
	return 0;
   }


int __declspec(dllexport) APIENTRY UserParamNames(char *data)
	{
	/* this function returns the name of the parameter requested */
	int i;
	i = (int) data[0];
	strcpy(data,"");
	if (i == 1) strcpy(data,"N0");
	if (i == 2) strcpy(data,"Nr2");
	if (i == 3) strcpy(data,"Nr4");
	if (i == 4) strcpy(data,"Nr6");
	if (i == 5) strcpy(data,"Nz1");
	if (i == 6) strcpy(data,"Nz2");
	if (i == 7) strcpy(data,"Nz3");
	if (i == 8) strcpy(data,"Nz4");
	return 0;
	}
