#include <windows.h>
#include <math.h>
#include <string.h>

/*
Written by Kenneth E. Moore
July 18, 2001
*/

int __declspec(dllexport) APIENTRY UserGrinDefinition(double *data);
int __declspec(dllexport) APIENTRY UserParamNames(char *data);

BOOL WINAPI DllMain (HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
	{
   return TRUE;
   }

/* the data is stored as follows:
ndata = the number of elements in the array data
data[0]  = the total number of values in the passed data array
data[1]  = x
data[2]  = y
data[3]  = z
data[4]  = wavelength in microns
data[5]  = millimeters per unit length (1.0 for mm, 25.4 for inches, 10.0 for cm and 1000.0 for meters)
data[6]  = n (to be computed by the dll and returned)
data[7]  = n * dn/dx (to be computed by the dll and returned)
data[8]  = n * dn/dy (to be computed by the dll and returned)
data[9]  = n * dn/dz (to be computed by the dll and returned)
data[10] = parameter 1 from user input
data[11] = parameter 2 from user input
etc... up to data[maxdata] where maxdata = int(data[0])

The DLL must compute data 6 through 9 given the other data,
and return 0 if it works; else return -1
*/

/* this DLL models a GRIN media similar to the GRIN 1 surface type, with symmetry about a torus axis. */
/* this is n(r) = N + A*t^2 where t is the distance to the center of the circlde defining the torus */

int __declspec(dllexport) APIENTRY UserGrinDefinition(double *data)
	{
   double r, r2, index, N, A, x, y, z, tx, ty, tz, R;

   N = data[10];
   A = data[11];
	R = data[12];
   x = data[1];
   y = data[2];
   z = data[3];
	r  = sqrt(x*x + z*z);

	/* this only could happen if r >= R */
	if (r == 0.0) return -1;

	tx = x*(1.0 - R/r);
	ty = y;
	tz = z*(1.0 - R/r);

	r2 = tx*tx + ty*ty + tz*tz;

   index = N + A*r2;
	data[6] = index;

   if (index < 1.0)
		{
		return -1;
		}

	/* now the derivatives, note they are multiplied by index */
   data[7] = 2.0*A*tx*index;
   data[8] = 2.0*A*ty*index;
   data[9] = 2.0*A*tz*index;
	return 0;
   }


int __declspec(dllexport) APIENTRY UserParamNames(char *data)
	{
	/* this function returns the name of the parameter requested */
	int i;
	i = (int) data[0];
	strcpy(data,"");
	if (i == 1) strcpy(data,"N0");
	if (i == 2) strcpy(data,"Nr2");
	if (i == 3) strcpy(data,"Torus R");
	return 0;
	}
